// Copyright 2024 Matthew Rich <matthewrich.conf@gmail.com>. All rights reserved.

package transport

import (
	"github.com/stretchr/testify/assert"
	"testing"
	"fmt"
	"os"
	"net/url"
	"path/filepath"
)

var TransportBufferTestFile = fmt.Sprintf("%s/foo", TempDir)

func TestNewTransportBufferReader(t *testing.T) {
	path := fmt.Sprintf("%s/foo", TempDir)
	u, e := url.Parse(fmt.Sprintf("file://%s", path))
	assert.Nil(t, e)

	writeErr := os.WriteFile(path, []byte("test"), 0644)
	assert.Nil(t, writeErr)

	file, err := NewBuffer(u)
	assert.Nil(t, err)
	assert.Equal(t, file.Path(), path)
}

func TestNewTransportBufferReaderExtension(t *testing.T) {
	u, e := url.Parse(fmt.Sprintf("file://%s.yaml", TransportBufferTestFile))
	assert.Nil(t, e)

	b := &Buffer{
		uri: u,
		path: filepath.Join(u.Hostname(), u.RequestURI()),
	}
	b.extension()
	assert.Equal(t, b.exttype, "yaml")
}
